package ru.dbotthepony.kommons.event

import java.util.function.Consumer

interface IUnitSubscripable : ISubscriptable<Unit> {
	fun addListener(listener: Runnable): ISubscriptable.L

	override fun addListener(listener: Consumer<Unit>): ISubscriptable.L {
		return addListener(Runnable { listener.accept(Unit) })
	}

	class Impl : IUnitSubscripable, Runnable {
		private inner class L(val callback: Runnable) : ISubscriptable.L {
			private var isRemoved = false

			init {
				subscribers.add(this)
			}

			override fun remove() {
				if (!isRemoved) {
					isRemoved = true
					queue.add(this)
				}
			}
		}

		private val subscribers = ArrayList<L>(0)
		private val queue = ArrayList<L>(0)

		override fun addListener(listener: Runnable): ISubscriptable.L {
			return L(listener)
		}

		override fun run() {
			queue.forEach { subscribers.remove(it) }
			queue.clear()
			subscribers.forEach { it.callback.run() }
		}
	}
}