package ru.dbotthepony.kommons.io

import java.io.DataInputStream
import java.io.DataOutputStream

class CollectionStreamCodec<E, C : MutableCollection<E>>(val elementCodec: IStreamCodec<E>, val collectionFactory: (Int) -> C) :
	IStreamCodec<C> {
	override fun read(stream: DataInputStream): C {
		val size = stream.readVarInt()

		if (size <= 0) {
			return collectionFactory.invoke(0)
		}

		val collection = collectionFactory.invoke(size)

		for (i in 0 until size) {
			collection.add(elementCodec.read(stream))
		}

		return collection
	}

	override fun write(stream: DataOutputStream, value: C) {
		stream.writeVarInt(value.size)
		value.forEach { elementCodec.write(stream, it) }
	}

	override fun copy(value: C): C {
		val new = collectionFactory.invoke(value.size)
		value.forEach { new.add(elementCodec.copy(it)) }
		return new
	}
}