package ru.dbotthepony.kommons.io

import java.io.DataInputStream
import java.io.DataOutputStream

/**
 * Represents value which can be encoded onto or decoded from stream.
 *
 * Also provides [copy] and [compare] methods
 */
interface IStreamCodec<V> {
	fun read(stream: DataInputStream): V
	fun write(stream: DataOutputStream, value: V)

	/**
	 * if value is immutable, return it right away
	 */
	fun copy(value: V): V

	/**
	 * Optional equality check override. Utilized to determine whenever e.g. network value is different from new value
	 *
	 * By default uses [Any.equals]
	 */
	fun compare(a: V, b: V): Boolean {
		return a == b
	}
}